# -*- coding: utf-8 -*-
from __future__ import absolute_import
import json

import requests


class PingdomGuiException(Exception):

    def __init__(self, http_response):
        content = json.loads(http_response.content)
        self.status_code = http_response.status_code
        self.status_desc = content['error']['statusdesc']
        self.error_message = content['error']['errormessage']
        super(PingdomGuiException, self).__init__(self.__str__())

    def __repr__(self):
        return 'pingdom.PingdomGuiException: HTTP `%s - %s` returned with message, "%s"' % \
               (self.status_code, self.status_desc, self.error_message)

    def __str__(self):
        return self.__repr__()


class Gui():

    def __init__(self, username, password):
        self.__username = username
        self.__password = password
        self.session = requests.session()
        _ua = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_6) AppleWebKit/537.36 " \
              + "(KHTML, like Gecko) Chrome/56.0.2924.87 Safari/537.36"
        self.headers = {
            "accept": "*/*",
            "accept-encoding": "gzip, deflate, br",
            "accept-language": "en-US,en;q=0.8",
            "cache-control": "no-cache",
            "content-type": "application/x-www-form-urlencoded; charset=UTF-8",
            "origin": "https://my.pingdom.com",
            "pragma": "no-cache",
            "referer": "https://my.pingdom.com/",
            "user-agent": _ua,
            "x-requested-with": "XMLHttpRequest"
        }

    def send(self, method, url, data=None, params=None):
        if data is None:
            data = {}
        if params is None:
            params = {}
        response = self.session.request(method, url, data=data, params=params, headers=self.headers)
        if response.status_code != 200:
            raise PingdomGuiException(response)
        return response

    def login(self):
        data = {"email": self.__username, "password": self.__password}
        self.send('post', 'https://my.pingdom.com/', data)



# -*- coding: utf-8 -*-
from __future__ import absolute_import


class Check(object):

    SKIP_ON_PRINT = ["cached_definition", "_id", "api"]
    SKIP_ON_JSON = [
        "api", "alert_policy_name", "cached_definition", "created", "lastresponsetime",
        "lasttesttime", "lasterrortime", "_id", "id", "status", "maintenanceids"
    ]

    def __init__(self, api, json=False, obj=False):
        self.api = api
        if json:
            self.from_json(json)
        elif obj:
            self.from_obj(obj)
        else:
            raise Exception("Missing check definition: use json or obj parameter")

    def __repr__(self):
        attr = ["{0}: {1} ".format(k, v) for k, v in self.__dict__.items() if k not in self.SKIP_ON_PRINT]
        return "\npingdom.Check <%s> \n  %s" % (self._id, "\n  ".join(attr))

    def __contains__(self, x):
        return x in self.__dict__

    def to_json(self):
        obj = {}
        for k, v in self.__dict__.items():

            if k in self.SKIP_ON_JSON:
                pass
            elif k == "tags" and len(v):
                obj["tags"] = ",".join(map(lambda x: x['name'], v))
            elif k == "requestheaders":
                i = 0
                for x, y in v.items():
                    obj["requestheader" + str(i)] = x + ":" + y
                    i = i + 1
            else:
                obj[k] = v
        return obj

    def from_json(self, obj):
        for k, v in obj.items():
            if k == "tags":
                self.tags = [x["name"] for x in v]
            if k == "name":
                self.name = v
            elif k == "hostname":
                self.host = v
            elif k == "id":
                self._id = v
            elif k == "status":
                self.paused = bool(v == "paused")
                self.status = v
            elif k == "type" and isinstance(k, dict):
                self.type = list(v.keys())[0]
                for x, y in v[self.type].items():
                    setattr(self, x, y)
            else:
                setattr(self, k, v)
        if 'tags' not in self:
            self.tags = []

    def from_obj(self, obj):
        for k, v in obj.items():
            setattr(self, k, v)
        if 'tags' not in self:
            self.tags = []

    def fetch(self):
        res = self.api.send('get', "checks", self._id)['check']
        self.from_json(res)



# -*- coding: utf-8 -*-
from __future__ import absolute_import

from .api import Api
from .gui import Gui
from .check import Check
from .maintenance import Maintenance


class Client(object):
    """Interact with API and GUI."""

    def __init__(self, username, password, apikey, email, api_version='2.0'):
        """
        Initializer.

        :param username: account main email
        :param password: account password
        :param apikey: Pingdom api key
        :param email: required for `Multi-User Authentication
                      <https://www.pingdom.com/resources/api#multi-user+authentication>`_.
        """
        self.username = username
        self.password = password
        self.apikey = apikey
        self.email = email
        self.api = Api(username, password, apikey, email, api_version)
        self.gui = Gui(username, password)
        # cache checks
        self.checks = {}
        for item in self.api.send('get', "checks", params={"include_tags": True})['checks']:
            self.checks[item["name"]] = Check(self.api, json=item)

    def get_check(self, name=None, _id=None):
        if name:
            return self.checks.get(name, None)
        elif _id:
            for _name, check in self.checks.items():
                if check._id == _id:
                    return check
        else:
            raise Exception("Missing name or _id")

    def get_checks(self, filters=None):
        if filters is None:
            return [c for c in self.checks.values()]
        return [c for c in self.checks.values() if not len(set(filters.get("tags", [])).intersection(set([x['name']
                for x in c.tags]))) == 0]

    def create_check(self, obj):
        c = Check(self.api, obj=obj)
        data = c.to_json()
        response = self.api.send(method='post', resource='checks', data=data)
        c._id = int(response["check"]["id"])
        c.from_json(self.api.send('get', "checks", response["check"]["id"])['check'])
        self.checks[c.name] = c
        return c

    def delete_check(self, check):
        if not check._id:
            raise Exception("CheckNotFound %s" % check.name)
        self.api.send(method='delete', resource='checks', resource_id=check._id)
        self.checks.pop(check.name, None)

    def update_check(self, check, changes):
        # ensure definition is updated
        check.fetch()
        # cache current definition to detect idempotence when modify is called
        cached_definition = check.to_json()
        check.from_obj(changes)
        data = check.to_json()
        if data == cached_definition:
            return False
        del data["type"]  # type can't be changed
        self.api.send(method='put', resource='checks', resource_id=check._id, data=data)
        check.from_json(self.api.send('get', "checks", check._id)['check'])
        return check

    def get_maintenances(self, filters=None):
        if filters is None:
            filters = {}
        self.gui.login()
        url = "https://my.pingdom.com/newims/maintenance/xhr?limit=10000&page_id=1&_=1489571119019"
        response = self.gui.send("get", url)
        res = []
        for obj in response.json()['events']:
            if "checks" in filters:
                wanted_ids = [check._id for check in filters['checks']]
                got_ids = [int(x['compound_id']) for x in obj['checks']]
                if len(set(wanted_ids).intersection(set(got_ids))) == 0:
                    continue
            window = Maintenance(self, json=obj)
            if "names" in filters and window.name not in filters['names']:
                continue
            if "after" in filters and filters["after"] >= window.start:
                continue
            if "before" in filters and filters["before"] <= window.stop:
                continue
            res.append(window)
        return res

    def create_maintenance(self, obj):
        window = Maintenance(self, obj=obj)
        self.gui.login()
        response = self.gui.send("post", "https://my.pingdom.com/ims/data/maintenance", window.to_json())
        window._id = response.json()["checks_maintenance"]["id"]
        return window

    def delete_maintenance(self, window):
        self.gui.login()
        self.gui.send("delete", "https://my.pingdom.com/ims/data/maintenance", params={"id": window._id})

    def servertime(self):
        return self.api.send(method='get', resource='servertime')['servertime']

    def get_summary_outage(self, checkid, start=None, end=None, order="asc"):
        params = {}
        if start is not None:
            params['from'] = start
        if end is not None:
            params['to'] = end
        if order is not None:
            params['order'] = order
        return self.api.send('get', resource="summary.outage", resource_id=checkid, params=params)



# -*- coding: utf-8 -*-
"""Tests for the Check class."""

from __future__ import absolute_import

import json
import os
import unittest

from pypingdom import check


def mock_data(path):
    """Return json data for mocking."""
    dir_path = os.path.dirname(os.path.realpath(__file__))
    resource_file = os.path.join(dir_path, os.path.normpath('resources'), path.lstrip('/'))
    return open(resource_file, mode='r').read()


class CheckTestCase(unittest.TestCase):
    """Test cases for the Check class."""

    def setUp(self):
        unittest.TestCase.setUp(self)
        self.checks = json.loads(mock_data('api/2.0/checks'))['checks']

    def test_check(self):
        """Test basic attributes."""
        acheck = check.Check(None, self.checks[0])
        self.assertEqual(acheck.name, self.checks[0]["name"])
        self.assertFalse("fubar" in acheck.tags)
        self.assertEqual(len(acheck.tags), len(self.checks[0]["tags"]))

    def test_check_notags(self):
        """Test that a check with no tags."""
        self.assertTrue("tags" not in self.checks[2])  # just to be sure ;-)
        acheck = check.Check(None, self.checks[2])
        self.assertTrue(isinstance(acheck.tags, list))
        self.assertEqual(len(acheck.tags), 0)



# -*- coding: utf-8 -*-
"""Tests for the client class."""

from __future__ import absolute_import

import os
import unittest

import requests_mock

from pypingdom import Client
from pypingdom.check import Check


def mock_data(path):
    """Return json data for mocking."""
    dir_path = os.path.dirname(os.path.realpath(__file__))
    resource_file = os.path.join(dir_path, os.path.normpath('resources'), path.lstrip('/'))
    return open(resource_file, mode='r').read()


class ClientTestCase(unittest.TestCase):
    """Test cases for the client methods."""

    @requests_mock.Mocker()
    def test_request(self, m):
        """Test a simple request."""
        base_url = "https://api.pingdom.com"
        fakepath = "/api/2.0/checks"
        m.request('get', base_url + fakepath, text=mock_data(fakepath))
        fakepath = "/api/2.0/servertime"
        m.request('get', base_url + fakepath, text=mock_data(fakepath))
        fakepath = "/api/2.0/summary.outage/85975"
        m.request('get', base_url + fakepath, text=mock_data(fakepath))

        client = Client(username="username",  # nosec
                        password="password",
                        apikey="apikey",
                        email="email")

        res = client.get_checks()
        self.assertEqual(len(res), 3)
        self.assertTrue(all(isinstance(x, Check) for x in res))
        check1 = client.get_check(name="My check 1")
        self.assertNotEqual(check1, None)
        check2 = client.get_check(name="My check 2")
        self.assertNotEqual(check1, check2)

        res = client.servertime()
        self.assertEqual(res, 1294237910)

        res = client.get_summary_outage("85975")
        self.assertTrue("summary" in res)
        self.assertTrue("states" in res['summary'])

    @requests_mock.Mocker()
    def test_get_checks(self, m):
        """Test a simple request."""
        base_url = "https://api.pingdom.com"
        fakepath = "/api/2.0/checks"
        m.request('get', base_url + fakepath, text=mock_data(fakepath))

        client = Client(username="username",  # nosec
                        password="password",
                        apikey="apikey",
                        email="email")

        res = client.get_checks()
        self.assertEqual(len(res), 3)
        self.assertTrue(all(isinstance(x, Check) for x in res))

        res = client.get_checks(filters={"tags": ['apache']})
        self.assertEqual(len(res), 1)



# -*- coding: utf-8 -*-
"""Tests for the api class."""

from __future__ import absolute_import

import os
import unittest

import requests_mock

from pypingdom.api import Api


def mock_data(path):
    """Return json data for mocking."""
    dir_path = os.path.dirname(os.path.realpath(__file__))
    resource_file = os.path.join(dir_path, os.path.normpath('resources'), path.lstrip('/'))
    return open(resource_file, mode='r').read()


class ApiTestCase(unittest.TestCase):
    """Test cases for the api methods."""

    @requests_mock.Mocker()
    def test_request(self, m):
        """Test a simple request."""
        api = Api(username="testuser", password="testpass", apikey="mocked")  # nosec
        m.request('get', api.base_url + "checks", text=mock_data('/api/2.0/checks'))
        res = api.send('get', "checks", params={"include_tags": True})
        self.assertTrue("checks" in res)



# -*- coding: utf-8 -*-
"""This is the pypingdom package."""

from __future__ import absolute_import
from pypingdom.client import Client  # noqa: F401
from pypingdom.check import Check  # noqa: F401
from pypingdom.maintenance import Maintenance  # noqa: F401

__author__ = 'Paolo Sechi <sekipaolo@gmail.com>'
__license__ = "Apache v2.0"



# -*- coding: utf-8 -*-
"""Well this is our setup tools definition for this project."""
import os

from setuptools import setup
# from setuptools.config import read_configuration  # only works in newer setuptools
# conf = read_configuration("setup.cfg")
#       long_description=conf['metadata']['long_description'],

BASEDIR = os.path.dirname(__file__)

with open(os.path.join(BASEDIR, 'README.rst'), 'r') as f:
    README = f.read()


setup(name='pypingdom',
      version="0.1.4",
      description='Client for Pingdom Services',
      long_description=README,
      author='Paolo Sechi',
      author_email='sekipaolo@gmail.com',
      install_requires=['requests>=0.10.8', 'six>=1.10.0'],
      setup_requires=['pytest-runner'],
      tests_require=['pytest>=3.0.7', 'requests-mock>=1.3.0'],
      url='https://github.com/sekipaolo/pypingdom',
      packages=['pypingdom'],
      license='Apache v2.0',
      platforms='Posix; MacOS X; Windows',
      zip_safe=True,
      classifiers=['Development Status :: 3 - Alpha',
                   'Intended Audience :: Developers',
                   'Intended Audience :: System Administrators',
                   'License :: OSI Approved :: Apache Software License',
                   'Operating System :: OS Independent',
                   'Topic :: System :: Monitoring',
                   'Programming Language :: Python',
                   'Programming Language :: Python :: 2.7',
                   'Programming Language :: Python :: 3.6'
                   ]
      )



#!/usr/sbin/env python
# -*- coding: utf-8 -*-
import yaml
import datetime

from six.moves import input

import pypingdom


with open("private_test_data.yml", 'r') as stream:
    auth = yaml.safe_load(stream)

client = pypingdom.Client(username=auth["username"],
                          password=auth["password"],
                          apikey=auth["apikey"],
                          email=auth["email"])

check_definition = {
    "name": "Ansible integration test",
    "paused": True,
    # "alert_policy": "",
    "type": "http",
    "host": "www.google.com",
    "url": "/",
    "requestheaders": {
        'XCustom': 'my header value'
    },
    "tags": ["pypingdom-test", "custom-tag"],
    "encryption": False
}


def clean():
    global client, check_definition
    checks = client.get_checks(filters={"tags": ["pypingdom-test"]})
    for check in checks:
        print("deleting check %s" % check.name)
        client.delete_check(check)
    checks = client.get_checks(filters={"tags": ["pypingdom-test"]})
    print(checks)
    assert checks == []  # nosec


def create_check():
    global client, check_definition
    c = client.create_check(check_definition)
    print("created %s" % c.name)
    check = client.get_check(check_definition["name"])
    assert check is not None  # nosec


def update_check():
    global client, check_definition
    check = client.get_check(check_definition["name"])
    print("updating check %s" % check.name)
    newcheck = client.update_check(check, {"requestheaders": {'XCustom': 'new header value'}})
    assert newcheck.requestheaders["XCustom"] == 'new header value'  # nosec


def delete_check():
    global client, check_definition
    check = client.get_check(check_definition["name"])
    print("deleting check %s" % check.name)
    client.delete_check(check)
    check = client.get_check(check_definition["name"])
    assert check is None  # nosec


def create_maintenance():
    global client, check_definition
    check = client.get_check(check_definition["name"])
    start = datetime.datetime.now() + datetime.timedelta(days=2)
    stop = start + datetime.timedelta(minutes=20)
    print("creating maintenance for check %s" % check.name)
    window = client.create_maintenance({
        "checks": [check],
        "name": "pypyngdom test maintenance",
        "start": start,
        "stop": stop
    })
    return window


def delete_maintenance(mid):
    check = client.get_check(check_definition["name"])
    for m in client.get_maintenances(filters={"checks": [check]}):
        print("deleting maintenance %s" % m.name)
        client.delete_maintenance(m)


create_check()
input("Continue?")
update_check()
input("Continue?")
create_maintenance()
input("Continue?")
delete_maintenance()
input("Continue?")
delete_check()
